pub struct DaryHeapI128 {
root: u128,
blocks: Vec<U128HeapBlock>,
len: usize,
backend: SimdBackend,
}Expand description
A cache-line-oriented 4-ary max-heap for large full-width 128-bit heaps. BinaryHeap can be faster for small heaps, monotone replacements, and heavily repeated keys.
Fields
root: u128blocks: Vec<U128HeapBlock>len: usizebackend: SimdBackendImplementations
Sourceimpl DaryHeapI128
impl DaryHeapI128
pub fn new() -> Self
pub fn with_capacity(capacity: usize) -> Self
pub fn len(&self) -> usize
pub fn is_empty(&self) -> bool
pub fn peek(&self) -> Option<i128>
pub fn push(&mut self, value: i128)
pub fn pop(&mut self) -> Option<i128>
Sourcepub fn replace(&mut self, value: i128) -> Option<i128>
pub fn replace(&mut self, value: i128) -> Option<i128>
Unconditionally replaces the greatest value, or inserts into an empty heap.
pub fn clear(&mut self)
pub fn into_sorted_vec(self) -> Vec<i128>
fn empty(capacity: usize, backend: SimdBackend) -> Self
fn build(values: Vec<i128>, backend: SimdBackend) -> Self
fn encode(value: i128) -> u128
fn decode(value: u128) -> i128
fn key(&self, index: usize) -> u128
fn set_key(&mut self, index: usize, value: u128)
fn sift_down_by<F>(&mut self, hole: usize, value: u128, max_index: F)
fn heapify(&mut self)
Sourceimpl DaryHeapI128
impl DaryHeapI128
fn sift_down_scalar(&mut self, hole: usize, value: u128)
unsafe fn sift_down_avx2(&mut self, hole: usize, value: u128)
unsafe fn sift_down_avx512(&mut self, hole: usize, value: u128)
fn sift_down(&mut self, hole: usize, value: u128)
fn sift_down_after_pop(&mut self, hole: usize, value: u128)