pub struct DaryHeapI64 {
root: u64,
blocks: Vec<HeapBlock<u64, 8>>,
len: usize,
backend: SimdBackend,
}Expand description
A cache-line-oriented 8-ary max-heap for large 64-bit heaps. BinaryHeap can be faster for small heaps and monotone replacements.
Fields
root: u64blocks: Vec<HeapBlock<u64, 8>>len: usizebackend: SimdBackendImplementations
Sourceimpl DaryHeapI64
impl DaryHeapI64
pub fn new() -> Self
pub fn with_capacity(capacity: usize) -> Self
pub fn len(&self) -> usize
pub fn is_empty(&self) -> bool
pub fn peek(&self) -> Option<i64>
pub fn push(&mut self, value: i64)
pub fn pop(&mut self) -> Option<i64>
Sourcepub fn replace(&mut self, value: i64) -> Option<i64>
pub fn replace(&mut self, value: i64) -> Option<i64>
Unconditionally replaces the greatest value, or inserts into an empty heap.
pub fn clear(&mut self)
pub fn into_sorted_vec(self) -> Vec<i64>
fn empty(capacity: usize, backend: SimdBackend) -> Self
fn build(values: Vec<i64>, backend: SimdBackend) -> Self
fn encode(value: i64) -> u64
fn decode(value: u64) -> i64
fn key(&self, index: usize) -> u64
fn set_key(&mut self, index: usize, value: u64)
fn sift_down_by<F>(&mut self, hole: usize, value: u64, max_index: F)
fn heapify(&mut self)
Sourceimpl DaryHeapI64
impl DaryHeapI64
fn sift_down_scalar(&mut self, hole: usize, value: u64)
unsafe fn sift_down_avx2(&mut self, hole: usize, value: u64)
unsafe fn sift_down_avx512(&mut self, hole: usize, value: u64)
fn sift_down(&mut self, hole: usize, value: u64)
fn sift_down_after_pop(&mut self, hole: usize, value: u64)