pub struct RadixHeapU64<T> {
buckets: [Vec<(u64, T)>; 65],
last: u64,
len: usize,
}Expand description
A min-priority queue whose removed keys are monotonically nondecreasing.
Values with equal keys have no specified removal order.
Fields
buckets: [Vec<(u64, T)>; 65]last: u64len: usizeImplementations
Sourceimpl<T> RadixHeapU64<T>
impl<T> RadixHeapU64<T>
pub fn new() -> Self
pub fn len(&self) -> usize
pub fn is_empty(&self) -> bool
Sourcepub fn push(&mut self, key: u64, value: T)
pub fn push(&mut self, key: u64, value: T)
Inserts a value whose key is not less than the key most recently removed.
Panics
Panics if key is less than the key most recently removed.
pub fn pop(&mut self) -> Option<(u64, T)>
pub fn clear(&mut self)
fn bucket_index(key: u64, last: u64) -> usize